from __future__ import print_function
import sys
import contextlib, time
import bcolz, numpy


PY2 = sys.version_info[0] == 2
if not PY2:
    xrange = range
    def range(*args):
        return list(xrange(*args))


@contextlib.contextmanager
def ctime(label=""):
    "Counts the time spent in some context"
    t = time.time()
    yield
    print(label, round(time.time() - t, 3), "sec")


N = 1000 * 1000

ct = bcolz.fromiter(((i, i*i, i*i*i) for i in xrange(N)), dtype='i8,i8,i8', count=N)

b = numpy.array(numpy.arange(N) % 2, dtype="bool")
c = bcolz.carray(b)

sorted_index = range(1, N, 2)
with ctime():
    r0 = (ct['f0'][sorted_index]).tolist()

with ctime():
    r1 = [x.f0 for x in ct.where(b)]
assert r0 == r1

with ctime():
    r2 = [x.f0 for x in ct.where(c)]
assert r0 == r2

with ctime():
    r3 = [x for x in ct['f0'].where(b)]
assert r0 == r3

with ctime():
    r4 = [x for x in ct['f0'].where(c)]
assert r0 == r4

# sum
with ctime("sum list"):
    r5 = sum([x for x in ct['f0'].where(c)])

with ctime("sum generator"):
    r6 = sum(x for x in ct['f0'].where(c))
assert r5 == r6

with ctime("sum method"):
    r7 = bcolz.fromiter((x for x in ct['f0'].where(c)),
                        dtype=ct['f0'].dtype,
                        count=c.wheretrue().sum()).sum()
assert r7 == r5

# sum with no NA's
with ctime("sum with no NA (list)"):
    r8 = sum([x for x in ct['f0'].where(c) if x == x])  # x==x check to leave out NA values

# sum with no NA's
with ctime("sum with no NA (generator)"):
    r9 = sum((x for x in ct['f0'].where(c) if x == x))  # x==x check to leave out NA values



# -*- coding: utf-8 -*-
########################################################################
#
#       License: BSD
#       Created: August 16, 2012
#       Author:  Francesc Alted - francesc@blosc.org
#
########################################################################

from __future__ import absolute_import

import os
import os.path
import json
from .py2help import dict_iteritems


ATTRSDIR = "__attrs__"


class attrs(object):
    """Accessor for attributes in carray/ctable objects.

    This class behaves very similarly to a dictionary, and attributes
    can be appended in the typical way::

       attrs['myattr'] = value

    And can be retrieved similarly::

       value = attrs['myattr']

    Attributes can be removed with::

       del attrs['myattr']

    This class also honors the `__iter__` and `__len__` special
    functions.  Moreover, a `getall()` method returns all the
    attributes as a dictionary.

    CAVEAT: The values should be able to be serialized with JSON for
    persistence.

    """

    def __init__(self, rootdir, mode, _new=False):
        self.rootdir = rootdir
        self.mode = mode
        self.attrs = {}

        if self.rootdir:
            self.attrsfile = os.path.join(self.rootdir, ATTRSDIR)

        if self.rootdir:
            if _new:
                self._create()
            else:
                self._open()

    def _create(self):
        if self.mode != 'r':
            # Empty the underlying file
            with open(self.attrsfile, 'wb') as rfile:
                rfile.write(json.dumps({}, ensure_ascii=True).encode('ascii'))
                rfile.write(b"\n")

    def _open(self):
        if not os.path.isfile(self.attrsfile):
            if self.mode != 'r':
                # Create a new empty file
                with open(self.attrsfile, 'wb') as rfile:
                    rfile.write("\n")
        # Get the serialized attributes
        with open(self.attrsfile, 'rb') as rfile:
            try:
                data = json.loads(rfile.read().decode('ascii'))
            except:
                raise IOError(
                    "Attribute file is not readable")
        self.attrs = data

    def _update_meta(self):
        """Update attributes on-disk."""
        if not self.rootdir:
            return
        with open(self.attrsfile, 'wb') as rfile:
            rfile.write(json.dumps(
                self.attrs, ensure_ascii=True).encode('ascii'))
            rfile.write(b"\n")

    def getall(self):
        return self.attrs.copy()

    def __getitem__(self, name):
        return self.attrs[name]

    def __setitem__(self, name, obj):
        if self.rootdir and self.mode == 'r':
            raise IOError(
                "Cannot modify an attribute when in 'r'ead-only mode")
        self.attrs[name] = obj
        self._update_meta()

    def __delitem__(self, name):
        """Remove the `name` attribute."""
        if self.rootdir and self.mode == 'r':
            raise IOError(
                "Cannot remove an attribute when in 'r'ead-only mode")
        del self.attrs[name]
        self._update_meta()

    def __iter__(self):
        return dict_iteritems(self.attrs)

    def __len__(self):
        return len(self.attrs)

    def __str__(self):
        if len(self.attrs) == 0:
            return "*no attrs*"
        fullrepr = ""
        for name in self.attrs:
            fullrepr += "%s : %s" % (name, self.attrs[name])
        return fullrepr

    def __repr__(self):
        if len(self.attrs) == 0:
            return str(self)
        fullrepr = ""
        for name in self.attrs:
            fullrepr += "%s : %r\n" % (name, self.attrs[name])
        return fullrepr



from time import time

import numpy as np

import bcolz


N = int(1e7)
CLEVEL = 5

a = np.linspace(0, 1, N)

t0 = time()
ac = bcolz.carray(a, cparams=bcolz.cparams(clevel=CLEVEL))
print("time creation (memory) ->", round(time() - t0, 3))
print("data (memory):", repr(ac))

t0 = time()
b = bcolz.carray(a, cparams=bcolz.cparams(clevel=CLEVEL),
                 rootdir='myarray', mode='w')
b.flush()
print("time creation (disk) ->", round(time() - t0, 3))
# print "meta (disk):", b.read_meta()

t0 = time()
an = np.array(a)
print("time creation (numpy) ->", round(time() - t0, 3))

t0 = time()
c = bcolz.carray(rootdir='myarray')
print("time open (disk) ->", round(time() - t0, 3))
#print "meta (disk):", c.read_meta()
print("data (disk):", repr(c))

t0 = time()
print(sum(ac))
print("time sum (memory, iter) ->", round(time() - t0, 3))

t0 = time()
print(sum(c))
print("time sum (disk, iter) ->", round(time() - t0, 3))

t0 = time()
print(bcolz.eval('sum(ac)'))
print("time sum (memory, eval) ->", round(time() - t0, 3))

t0 = time()
print(bcolz.eval('sum(c)'))
print("time sum (disk, eval) ->", round(time() - t0, 3))

t0 = time()
print(ac.sum())
print("time sum (memory, method) ->", round(time() - t0, 3))

t0 = time()
print(c.sum())
print("time sum (disk, method) ->", round(time() - t0, 3))

t0 = time()
print(a.sum())
print("time sum (numpy, method) ->", round(time() - t0, 3))



import os
from conans import ConanFile, CMake, tools


class CbloscConan(ConanFile):
    name = "c-blosc"
    description = "An extremely fast, multi-threaded, meta-compressor library"
    license = "BSD"
    url = "https://github.com/Blosc/c-blosc"
    settings = "os", "compiler", "build_type", "arch"
    options = {"shared": [True, False]}
    default_options = "shared=False"
    generators = "cmake"
    exports_sources = "*", "!test_package/*", "!appveyor*", "!.*.yml", "!*.py", "!.*"

    @property
    def run_tests(self):
        return "CONAN_RUN_TESTS" in os.environ

    def build(self):
        os.mkdir("build")
        tools.replace_in_file("CMakeLists.txt", "project(blosc)", '''project(blosc)
            include(${CMAKE_BINARY_DIR}/../conanbuildinfo.cmake)
            conan_basic_setup(NO_OUTPUT_DIRS)''')
        cmake = CMake(self)
        cmake.definitions["BUILD_TESTS"] = "ON" if self.run_tests else "OFF"
        cmake.definitions["BUILD_BENCHMARKS"] = "ON" if self.run_tests else "OFF"
        cmake.definitions["BUILD_SHARED"] = "ON" if (self.options.shared or self.run_tests) else "OFF"
        cmake.definitions["BUILD_STATIC"] = "OFF" if self.options.shared else "ON"
        cmake.configure(build_folder="build")
        cmake.build()

        if self.run_tests:
            self.output.warn("Running tests!!")
            self.launch_tests()

    def launch_tests(self):
        """Conan will remove rpaths from shared libs to be able to reuse the shared libs, we need
        to tell the tests where to find the shared libs"""
        test_args = "-VV" if tools.os_info.is_windows else ""
        with tools.chdir("build"):
            outdir = os.path.join(self.build_folder, "build", "blosc")
            if tools.os_info.is_macos:
                prefix = "DYLD_LIBRARY_PATH=%s" % outdir
            elif tools.os_info.is_windows:
                prefix = "PATH=%s;%%PATH%%" % outdir
            elif tools.os_info.is_linux:
                prefix = "LD_LIBRARY_PATH=%s" % outdir
            else:
                return
            self.run("%s ctest %s" % (prefix, test_args))

    def package(self):
        self.copy("blosc.h", dst="include", src="blosc")
        self.copy("blosc-export.h", dst="include", src="blosc")
        self.copy("*libblosc.a", dst="lib", keep_path=False)

        if self.options.shared:
            self.copy("*/blosc.lib", dst="lib", keep_path=False)
            self.copy("*blosc.dll", dst="bin", keep_path=False)
            self.copy("*blosc.*dylib*", dst="lib", keep_path=False, symlinks=True)
            self.copy("*blosc.so*", dst="lib", keep_path=False, symlinks=True)
            self.copy("*libblosc.dll.a", dst="lib", keep_path=False) # Mingw
        else:
            self.copy("*libblosc.lib", dst="lib", src="", keep_path=False)

    def package_info(self):
        if self.settings.compiler == "Visual Studio" and not self.options.shared:
            self.cpp_info.libs = ["libblosc"]
        else:
            self.cpp_info.libs = ["blosc"]
        if self.settings.os == "Linux":
            self.cpp_info.libs.append("pthread")



# Benchmark to compare the times for evaluating queries.
# Numexpr is needed in order to execute this.

import math
from time import time

import numpy as np

import bcolz


N = int(1e8)  # the number of elements in x
clevel = 9    # the compression level
cname = "blosclz"  # the compressor name
sexpr = "(x+1)<10"  # small number of items
# sexpr = "(x+1)<1000000"              # large number
# sexpr = "(2*x*x*x+.3*y**2+z+1)<10"  # small number
#sexpr = "(2*x*x*x+.3*y**2+z+1)<1e15"  # medium number
#sexpr = "(2*x*x*x+.3*y**2+z+1)<1e20"  # large number

print("Creating inputs...")

cparams = bcolz.cparams(clevel=clevel, cname=cname)

x = np.arange(N)
cx = bcolz.carray(x, cparams=cparams)
if 'y' not in sexpr:
    t = bcolz.ctable((cx,), names=['x'])
else:
    y = np.arange(N)
    z = np.arange(N)
    cy = bcolz.carray(y, cparams=cparams)
    cz = bcolz.carray(z, cparams=cparams)
    t = bcolz.ctable((cx, cy, cz), names=['x', 'y', 'z'])
nt = t[:]

print("Querying '%s' with 10^%d points" % (sexpr, int(math.log10(N))))

t0 = time()
out = [r for r in x[eval(sexpr)]]
print("Time for numpy--> %.3f" % (time() - t0,))

t0 = time()
out = [r for r in t[eval(sexpr)]]
print("Time for structured array--> %.3f" % (time() - t0,))

t0 = time()
out = [r for r in cx[sexpr]]
print("Time for carray --> %.3f" % (time() - t0,))

# Uncomment the next for disabling threading
#ne.set_num_threads(1)
#bcolz.blosc_set_num_threads(1)
# Seems that this works better if we dividw the number of cores by 2.
# Maybe due to some contention between Numexpr and Blosc?
#bcolz.set_num_threads(bcolz.ncores//2)

t0 = time()
#cout = t[t.eval(sexpr, cparams=cparams)]
cout = [r for r in t.where(sexpr)]
#cout = [r['x'] for r in t.where(sexpr)]
#cout = [r['y'] for r in t.where(sexpr, colnames=['x', 'y'])]
print("Time for ctable--> %.3f" % (time() - t0,))
print("cout-->", len(cout), cout[:10])

#assert_array_equal(out, cout, "Arrays are not equal")



from __future__ import print_function

import itertools
import numpy as np
import numexpr as ne
import bcolz
import time
import cProfile
import inspect

print("numexpr version:", ne.__version__)
bcolz.defaults.cparams['shuffle'] = bcolz.SHUFFLE
#bcolz.defaults.cparams['shuffle'] = bcolz.BITSHUFFLE
bcolz.defaults.cparams['cname'] = 'blosclz'
#bcolz.defaults.cparams['cname'] = 'lz4'
bcolz.defaults.cparams['clevel'] = 5
#bcolz.defaults.vm = "dask"
#bcolz.defaults.vm = "python"
bcolz.defaults.vm = "numexpr"

N = 1e8
LMAX = 1e3
npa = np.arange(N)
npb = np.arange(N)
ct = bcolz.ctable([npa, npb], names=["a", "b"])


def do_cprofile(func):
    def profiled_func(*args, **kwargs):
        profile = cProfile.Profile()
        try:
            profile.enable()
            result = func(*args, **kwargs)
            profile.disable()
            return result
        finally:
            profile.print_stats(sort='cumulative')
    return profiled_func



def timefunc(f):
    def f_timer(*args, **kwargs):
        start = time.time()
        result = f(*args, **kwargs)
        end = time.time()
        print(f.__name__, 'took', round(end - start, 3), 'sec')
        return result
    return f_timer


@timefunc
def where_numpy():
    return sum(npa[i] for i in np.where((npa > 5) & (npb < LMAX))[0])

@timefunc
def where_numexpr():
    return sum(npa[i] for i in np.where(
        ne.evaluate('(npa > 5) & (npb < LMAX)'))[0])

@timefunc
#@do_cprofile
def bcolz_where():
    return sum(r.a for r in ct.where("(a > 5) & (b < LMAX)"))

@timefunc
#@do_cprofile
def bcolz_where_numpy():
    return sum(r.a for r in ct.where("(npa > 5) & (npb < LMAX)"))

@timefunc
#@do_cprofile
def bcolz_where_numexpr():
    return sum(r.a for r in ct.where(ne.evaluate("(npa > 5) & (npb < LMAX)")))

@timefunc
#@do_cprofile
def whereblocks():
    sum = 0.
    for r in ct.whereblocks("(a > 5) & (b < LMAX)", blen=None):
        sum += r['a'].sum()
    return sum

@timefunc
#@do_cprofile
def fetchwhere_bcolz():
    return ct.fetchwhere("(a > 5) & (b < LMAX)", out_flavor='bcolz')['a'].sum()

@timefunc
#@do_cprofile
def fetchwhere_numpy():
    return ct.fetchwhere("(a > 5) & (b < LMAX)", out_flavor='numpy')['a'].sum()

@timefunc
#@do_cprofile
def fetchwhere_dask():
    result = ct.fetchwhere("(a > 5) & (b < LMAX)", vm="dask")['a'].sum()
    return result


print(repr(ct))

a0 = where_numpy()
print("a0:", a0)
a1 = where_numexpr()
assert a0 == a1
a1 = bcolz_where()
assert a0 == a1
a1 = bcolz_where_numpy()
assert a0 == a1
a1 = bcolz_where_numexpr()
assert a0 == a1
a1 = whereblocks()
assert a0 == a1
a1 = fetchwhere_bcolz()
assert a0 == a1
a1 = fetchwhere_numpy()
assert a0 == a1
a1 = fetchwhere_dask()
assert a0 == a1



# Benchmark for getitem

from time import time

import numpy as np

import bcolz
from bcolz.py2help import xrange


N = int(1e7)  # the number of elements in x
M = 100000  # the elements to get
clevel = 1  # the compression level

print("Creating inputs with %d elements..." % N)

cparams = bcolz.cparams(clevel)

# x = np.arange(N)
x = np.zeros(N, dtype="f8")
y = x.copy()
z = x.copy()
cx = bcolz.carray(x, cparams=cparams)
cy = cx.copy()
cz = cx.copy()
ct = bcolz.ctable((cx, cy, cz), names=['x', 'y', 'z'])
t = ct[:]

print("Starting benchmark now for getting %d elements..." % M)
# Retrieve from a ndarray
t0 = time()
vals = [x[i] for i in xrange(0, M, 3)]
print("Time for array--> %.3f" % (time() - t0,))
print("vals-->", len(vals))

#bcolz.set_num_threads(bcolz.ncores//2)

# Retrieve from a carray
t0 = time()
cvals = [cx[i] for i in xrange(0, M, 3)]
#cvals = cx[:M:3][:].tolist()
print("Time for carray--> %.3f" % (time() - t0,))
print("vals-->", len(cvals))
assert vals == cvals

# Retrieve from a structured ndarray
t0 = time()
vals = [t[i] for i in xrange(0, M, 3)]
print("Time for structured array--> %.3f" % (time() - t0,))
print("vals-->", len(vals))

# Retrieve from a ctable
t0 = time()
cvals = [ct[i] for i in xrange(0, M, 3)]
#cvals = ct[:M:3][:].tolist()
print("Time for ctable--> %.3f" % (time() - t0,))
print("vals-->", len(cvals))
assert vals == cvals



########################################################################
#
#       License: BSD
#       Created: August 05, 2010
#       Author:  Francesc Alted - francesc@blosc.org
#
########################################################################

"""Here are some definitions for some C headers dependencies."""

import sys

# Standard C functions.
cdef extern from "stdlib.h":
    ctypedef long size_t
    ctypedef long uintptr_t
    void *malloc(size_t size)
    void *realloc(void *ptr, size_t size)
    void free(void *ptr)

cdef extern from "string.h":
    char *strchr(char *s, int c)
    char *strcpy(char *dest, char *src)
    char *strncpy(char *dest, char *src, size_t n)
    int strcmp(char *s1, char *s2)
    char *strdup(char *s)
    void *memcpy(void *dest, void *src, size_t n)
    void *memset(void *s, int c, size_t n)

cdef extern from "time.h":
    ctypedef int time_t


#-----------------------------------------------------------------------------

# Some helper routines from the Python API
# PythonHelper.h is used to help make
# python 2 and 3 both work.
cdef extern from "PythonHelper.h":

    # special types
    ctypedef int Py_ssize_t

    # references
    void Py_INCREF(object)
    void Py_DECREF(object)

    # To release global interpreter lock (GIL) for threading
    void Py_BEGIN_ALLOW_THREADS()
    void Py_END_ALLOW_THREADS()

    # Functions for integers
    object PyInt_FromLong(long)
    long PyInt_AsLong(object)
    object PyLong_FromLongLong(long long)
    long long PyLong_AsLongLong(object)

    # Functions for floating points
    object PyFloat_FromDouble(double)

    # Functions for strings
    object PyBytes_FromString(char *)
    object PyBytes_FromStringAndSize(char *s, int len)
    char *PyBytes_AsString(object string)
    char *PyBytes_AS_STRING(object string)
    size_t PyBytes_GET_SIZE(object string)

    # Functions for lists
    int PyList_Append(object list, object item)

    # Functions for tuples
    object PyTuple_New(int)
    int PyTuple_SetItem(object, int, object)
    object PyTuple_GetItem(object, int)
    int PyTuple_Size(object tuple)

    # Functions for dicts
    int PyDict_Contains(object p, object key)
    object PyDict_GetItem(object p, object key)

    # Functions for objects
    object PyObject_GetItem(object o, object key)
    int PyObject_SetItem(object o, object key, object v)
    int PyObject_DelItem(object o, object key)
    long PyObject_Length(object o)
    int PyObject_Compare(object o1, object o2)
    int PyObject_AsReadBuffer(object obj, void **buffer, Py_ssize_t *buffer_len)

    # Functions for buffers
    object PyBuffer_FromMemory(void *ptr, Py_ssize_t size)

    ctypedef unsigned int Py_uintptr_t


#-----------------------------------------------------------------------------


## Local Variables:
## mode: python
## tab-width: 4
## fill-column: 78
## End:



# Benchmark that compares the times for concatenating arrays with
# compressed arrays vs plain numpy arrays.  The 'numpy' and 'concat'
# styles are for regular numpy arrays, while 'carray' is for carrays.
#
# Call this benchmark as:
#
# python bench/concat.py style
#
# where `style` can be any of 'numpy', 'concat' or 'bcolsz'
#
# You can modify other parameters from the command line if you want:
#
# python bench/concat.py style arraysize nchunks nrepeats clevel
#

from __future__ import absolute_import

import sys
import math
import time

import numpy

import bcolz
from bcolz.py2help import xrange
from .bench_helper import ctime


def concat(data):
    tlen = sum(x.shape[0] for x in data)
    alldata = numpy.empty((tlen,))
    pos = 0
    for x in data:
        step = x.shape[0]
        alldata[pos:pos + step] = x
        pos += step

    return alldata


def append(data, clevel):
    alldata = bcolz.carray(data[0], cparams=bcolz.cparams(clevel))
    for carr in data[1:]:
        alldata.append(carr)

    return alldata


class Suite:
    a = None
    N = 1000000
    K = 10
    T = 3
    clevel = 1
    style = 'bcolz'
    r = None

    def __init__(self, N=1000000, K=10, T=3, clevel=1, style='bcolz'):
        Suite.N = N
        Suite.K = K
        Suite.T = T
        Suite.clevel = clevel
        Suite.style = style
        Suite.r = None

    def setup(self):
        # The next datasets allow for very high compression ratios
        Suite.a = [numpy.arange(Suite.N, dtype='f8') for _ in range(Suite.K)]
        print("problem size: (%d) x %d = 10^%g" % (Suite.N, Suite.K,
                                                   math.log10(Suite.N * Suite.K)))

    def time_concatenate(self):
        if Suite.style == 'numpy':
            for _ in xrange(Suite.T):
                Suite.r = numpy.concatenate(Suite.a, 0)
        elif Suite.style == 'concat':
            for _ in xrange(Suite.T):
                Suite.r = concat(Suite.a)
        elif Suite.style == 'bcolz':
            for _ in xrange(Suite.T):
                Suite.r = append(Suite.a, Suite.clevel)

    def print_container_size(self):
        if Suite.style == 'bcolz':
            size = Suite.r.cbytes
        else:
            size = Suite.r.size * Suite.r.dtype.itemsize
        print("size of the final container: %.3f MB" %
              (size / float(1024 * 1024)))


if __name__ == '__main__':
    if len(sys.argv) < 2:
        print(
            "Pass at least one of these styles: 'numpy', 'concat' or 'bcolz' ")
        sys.exit(1)

    style = sys.argv[1]
    if len(sys.argv) == 2:
        N, K, T, clevel = (1000000, 10, 3, 1)
    else:
        N, K, T = [int(arg) for arg in sys.argv[2:5]]
        if len(sys.argv) > 5:
            clevel = int(sys.argv[5])
        else:
            clevel = 0

    # run benchmark
    suite = Suite(N=N, K=K, T=T, clevel=clevel, style=style)
    suite.setup()
    with ctime("time_concatenate"):
        suite.time_concatenate()
    suite.print_container_size()



# Benchmark for iterators

from time import time

import numpy as np

import bcolz


N = 1e8  # the number of elements in x
clevel = 5  # the compression level
sexpr = "(x-1) < 10."  # the expression to compute
# sexpr = "((x-1) % 1000) == 0."  # the expression to compute
#sexpr = "(2*x**3+.3*y**2+z+1)<0"  # the expression to compute

cparams = bcolz.cparams(clevel)

print("Creating inputs...")

x = np.arange(N)
cx = bcolz.carray(x, cparams=cparams)
if 'y' not in sexpr:
    ct = bcolz.ctable((cx,), names=['x'])
else:
    y = np.arange(N)
    z = np.arange(N)
    cy = bcolz.carray(y, cparams=cparams)
    cz = bcolz.carray(z, cparams=cparams)
    ct = bcolz.ctable((cx, cy, cz), names=['x', 'y', 'z'])

print("Evaluating...", sexpr)
t0 = time()
cbout = ct.eval(sexpr)
print("Time for evaluation--> %.3f" % (time() - t0,))
print("Converting to numy arrays")
bout = cbout[:]
t = ct[:]

t0 = time()
cbool = bcolz.carray(bout, cparams=cparams)
print("Time for converting boolean--> %.3f" % (time() - t0,))
print("cbool-->", repr(cbool))

t0 = time()
vals = [v for v in cbool.wheretrue()]
print("Time for wheretrue()--> %.3f" % (time() - t0,))
print("vals-->", len(vals))

print("Starting benchmark now...")
# Retrieve from a ndarray
t0 = time()
vals = [v for v in x[bout]]
print("Time for array--> %.3f" % (time() - t0,))
#print("vals-->", len(vals))

#bcolz.set_num_threads(bcolz.ncores//2)

# Retrieve from a carray
t0 = time()
#cvals = [v for v in cx[cbout]]
cvals = [v for v in cx.where(cbout)]
print("Time for carray--> %.3f" % (time() - t0,))
#print("vals-->", len(cvals))
assert vals == cvals

# Retrieve from a structured ndarray
t0 = time()
vals = [tuple(v) for v in t[bout]]
print("Time for structured array--> %.3f" % (time() - t0,))
#print("vals-->", len(vals))

# Retrieve from a ctable
t0 = time()
#cvals = [tuple(v) for v in ct[cbout]]
cvals = [v for v in ct.where(cbout)]
print("Time for ctable--> %.3f" % (time() - t0,))
#print("vals-->", len(cvals))
assert vals == cvals



import numpy as np
import bcolz
from .bench_helper import ctime


class Suite:

    def setup(self):
        self.N = 1e8
        self.dtype = 'i4'
        self.start = 5
        self.stop = self.N
        self.step = 4
        self.a = np.arange(self.start, self.stop, self.step, dtype=self.dtype)

    def time_arange(self):
        ac = bcolz.arange(self.start, self.stop, self.step, dtype=self.dtype)

if __name__ == '__main__':
    suite = Suite()
    suite.setup()
    with ctime("time_arange"):
        suite.time_arange()



from conans import ConanFile, CMake, tools


class CbloscTestConan(ConanFile):
    settings = "os", "compiler", "build_type", "arch"
    generators = "cmake"

    def build(self):
        cmake = CMake(self)
        cmake.configure()
        cmake.build()

    def imports(self):
        self.copy("*.dll", dst="bin", src="bin" )
        self.copy("*.dylib*", dst="bin", src="lib")
        self.copy("*.so*", dst="bin", src="lib")

    def test(self):
        with tools.chdir("bin"):
            if tools.os_info.is_windows:
                self.run("example")
            else:
                prefix = "DYLD_LIBRARY_PATH=." if tools.os_info.is_macos else ""
                self.run("%s ./example" % prefix)



import numpy as np
import bcolz
import time


bcolz.defaults.cparams['shuffle'] = bcolz.SHUFFLE
# bcolz.defaults.cparams['shuffle'] = bcolz.BITSHUFFLE
bcolz.defaults.cparams['cname'] = 'blosclz'
# bcolz.defaults.cparams['cname'] = 'lz4'
bcolz.defaults.cparams['clevel'] = 9

N = int(1e8)
a = np.arange(N)
ca = bcolz.carray(a)


def timefunc(f):
    def f_timer(*args, **kwargs):
        start = time.time()
        result = f(*args, **kwargs)
        end = time.time()
        print(f.__name__, 'took', round(end - start, 3), 'sec')
        return result
    return f_timer


@timefunc
def iterblocks0():
    return a.sum()


@timefunc
def iterblocks1(arr):
    return sum(i for i in arr)


@timefunc
def iterblocks2(arr):
    sum_ = 0.
    for b in bcolz.iterblocks(arr, blen=arr.chunklen):
        sum_ += b.sum()
    return sum_


print(repr(ca))

a0 = iterblocks0()
print("a0:", a0)
# a1 = iterblocks1(ca)
# assert a0 == a1
a1 = iterblocks2(ca)
assert a0 == a1



########################################################################
#
# License: BSD
#       Created: August 16, 2012
#       Author:  Francesc Alted - francesc@blosc.org
#
########################################################################

from __future__ import absolute_import

from sys import version_info as v

# Check this Python version is supported
if any([v < (2, 7), (3,) < v < (3, 5)]):
    raise Exception("Unsupported Python version %d.%d. Requires Python >= 2.7 "
                    "or >= 3.5." % v[:2])

import os
from glob import glob
import sys

from setuptools import setup, Extension, find_packages
from pkg_resources import resource_filename

# For guessing the capabilities of the CPU for C-Blosc
try:
    # Currently just Intel and some ARM archs are supported by cpuinfo module
    import cpuinfo
    cpu_info = cpuinfo.get_cpu_info()
except:
    cpu_info = {'flags': []}


class LazyCommandClass(dict):
    """
    Lazy command class that defers operations requiring Cython and numpy until
    they've actually been downloaded and installed by setup_requires.
    """
    def __contains__(self, key):
        return (
            key == 'build_ext'
            or super(LazyCommandClass, self).__contains__(key)
        )

    def __setitem__(self, key, value):
        if key == 'build_ext':
            raise AssertionError("build_ext overridden!")
        super(LazyCommandClass, self).__setitem__(key, value)

    def __getitem__(self, key):
        if key != 'build_ext':
            return super(LazyCommandClass, self).__getitem__(key)

        from Cython.Distutils import build_ext as cython_build_ext

        class build_ext(cython_build_ext):
            """
            Custom build_ext command that lazily adds numpy's include_dir to
            extensions.
            """
            def build_extensions(self):
                """
                Lazily append numpy's include directory to Extension includes.

                This is done here rather than at module scope because setup.py
                may be run before numpy has been installed, in which case
                importing numpy and calling `numpy.get_include()` will fail.
                """
                numpy_incl = resource_filename('numpy', 'core/include')
                for ext in self.extensions:
                    ext.include_dirs.append(numpy_incl)

                # This explicitly calls the superclass method rather than the
                # usual super() invocation because distutils' build_class, of
                # which Cython's build_ext is a subclass, is an old-style class
                # in Python 2, which doesn't support `super`.
                cython_build_ext.build_extensions(self)
        return build_ext


# Global variables
CFLAGS = os.environ.get('CFLAGS', '').split()
LFLAGS = os.environ.get('LFLAGS', '').split()
# Allow setting the Blosc dir if installed in the system
BLOSC_DIR = os.environ.get('BLOSC_DIR', '')

# Sources & libraries
inc_dirs = ['bcolz']
lib_dirs = []
libs = []
def_macros = []
sources = ['bcolz/carray_ext.pyx']

optional_libs = []

# Handle --blosc=[PATH] --lflags=[FLAGS] --cflags=[FLAGS]
args = sys.argv[:]
for arg in args:
    if arg.find('--blosc=') == 0:
        BLOSC_DIR = os.path.expanduser(arg.split('=')[1])
        sys.argv.remove(arg)
    if arg.find('--lflags=') == 0:
        LFLAGS = arg.split('=')[1].split()
        sys.argv.remove(arg)
    if arg.find('--cflags=') == 0:
        CFLAGS = arg.split('=')[1].split()
        sys.argv.remove(arg)

if BLOSC_DIR != '':
    # Using the Blosc library
    lib_dirs += [os.path.join(BLOSC_DIR, 'lib')]
    inc_dirs += [os.path.join(BLOSC_DIR, 'include')]
    libs += ['blosc']
else:
    # Compiling everything from sources
    sources += [f for f in glob('c-blosc/blosc/*.c')
                if 'avx2' not in f and 'sse2' not in f]
    sources += glob('c-blosc/internal-complibs/lz4*/*.c')
    sources += glob('c-blosc/internal-complibs/snappy*/*.cc')
    sources += glob('c-blosc/internal-complibs/zlib*/*.c')
    sources += glob('c-blosc/internal-complibs/zstd*/*/*.c')
    inc_dirs += [os.path.join('c-blosc', 'blosc')]
    inc_dirs += [d for d in glob('c-blosc/internal-complibs/*')
                 if os.path.isdir(d)]
    inc_dirs += [d for d in glob('c-blosc/internal-complibs/zstd*/*')
                 if os.path.isdir(d)]
    def_macros += [('HAVE_LZ4', 1), ('HAVE_SNAPPY', 1), ('HAVE_ZLIB', 1),
                   ('HAVE_ZSTD', 1)]

    # Guess SSE2 or AVX2 capabilities
    # SSE2
    if 'DISABLE_BCOLZ_SSE2' not in os.environ and 'sse2' in cpu_info['flags']:
        print('SSE2 detected')
        CFLAGS.append('-DSHUFFLE_SSE2_ENABLED')
        sources += [f for f in glob('c-blosc/blosc/*.c') if 'sse2' in f]
        if os.name == 'posix':
            CFLAGS.append('-msse2')
        elif os.name == 'nt':
            def_macros += [('__SSE2__', 1)]

    # AVX2
    if 'DISABLE_BCOLZ_AVX2' not in os.environ and 'avx2' in cpu_info['flags']:
        print('AVX2 detected')
        CFLAGS.append('-DSHUFFLE_AVX2_ENABLED')
        sources += [f for f in glob('c-blosc/blosc/*.c') if 'avx2' in f]
        if os.name == 'posix':
            CFLAGS.append('-mavx2')
        elif os.name == 'nt':
            def_macros += [('__AVX2__', 1)]


tests_require = []
if v < (3,):
    tests_require.extend(['unittest2', 'mock'])

# compile and link code instrumented for coverage analysis
if os.getenv('TRAVIS') and os.getenv('CI') and v[0:2] == (2, 7):
    CFLAGS.extend(["-fprofile-arcs", "-ftest-coverage"])
    LFLAGS.append("-lgcov")

setup(
    name="bcolz",
    use_scm_version={
        'version_scheme': 'guess-next-dev',
        'local_scheme': 'dirty-tag',
        'write_to': 'bcolz/version.py'
    },
    description='columnar and compressed data containers.',
    long_description="""\

bcolz provides columnar and compressed data containers.  Column
storage allows for efficiently querying tables with a large number of
columns.  It also allows for cheap addition and removal of column.  In
addition, bcolz objects are compressed by default for reducing
memory/disk I/O needs.  The compression process is carried out
internally by Blosc, a high-performance compressor that is optimized
for binary data.

""",
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'Intended Audience :: Information Technology',
        'Intended Audience :: Science/Research',
        'License :: OSI Approved :: BSD License',
        'Programming Language :: Python',
        'Topic :: Software Development :: Libraries :: Python Modules',
        'Operating System :: Microsoft :: Windows',
        'Operating System :: Unix',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.6',
    ],
    author='Francesc Alted',
    author_email='francesc@blosc.org',
    maintainer='Francesc Alted',
    maintainer_email='francesc@blosc.org',
    url='https://github.com/Blosc/bcolz',
    license='BSD',
    platforms=['any'],
    ext_modules=[
        Extension(
            'bcolz.carray_ext',
            include_dirs=inc_dirs,
            define_macros=def_macros,
            sources=sources,
            library_dirs=lib_dirs,
            libraries=libs,
            extra_link_args=LFLAGS,
            extra_compile_args=CFLAGS
        )
    ],
    install_requires=['numpy>=1.7'],
    setup_requires=[
        'cython>=0.22',
        'numpy>=1.7',
        'setuptools>18.0',
        'setuptools-scm>1.5.4'
    ],
    tests_require=tests_require,
    extras_require=dict(
        optional=[
            'numexpr>=2.5.2',
            'dask>=0.9.0',
            'pandas',
            'tables'
        ],
        test=tests_require
    ),
    packages=find_packages(),
    package_data={'bcolz': ['carray_ext.pxd']},
    cmdclass=LazyCommandClass(),
)



# Benchmark to compare times for iterators in generator contexts by
# using carrays vs plain numpy arrays.

from time import time

import numpy as np

import bcolz


N = int(1e6)
bcolz.defaults.cparams['cname'] = 'blosclz'
bcolz.defaults.cparams['clevel'] = 9

a = np.arange(N)
b = bcolz.carray(a)

t0 = time()
# sum1 = sum(a)
sum1 = sum((v for v in a[2::3] if v < 10))
t1 = time() - t0
print("Summing using numpy iterator: %.3f" % t1)

t0 = time()
#sum2 = sum(b)
sum2 = sum((v for v in b.iter(2, None, 3) if v < 10))
t2 = time() - t0
print("Summing using carray iterator: %.3f  speedup: %.2f" % (t2, t1 / t2))

assert sum1 == sum2, "Summations are not equal!"



# Benchmark to compare the times for computing expressions by using
# eval() on carray/numpy arrays.  Numexpr is needed in order to
# execute this.

from __future__ import print_function

import math
from time import time

import numpy as np
import numexpr as ne

import bcolz


N = 1e7  # the number of elements in x
clevel = 9  # the compression level
sexprs = ["(x+1)<0",
          "(2*x**2+.3*y**2+z+1)<0",
          "((.25*x + .75)*x - 1.5)*x - 2",
          "(((.25*x + .75)*x - 1.5)*x - 2)<0",
]

# Initial dataset
# x = np.arange(N)
x = np.linspace(0, 100, N)

doprofile = False


def compute_ref(sexpr):
    t0 = time()
    out = eval(sexpr)
    print("Time for plain numpy --> %.3f" % (time() - t0,))

    t0 = time()
    out = ne.evaluate(sexpr)
    print("Time for numexpr (numpy) --> %.3f" % (time() - t0,))


def compute_carray(sexpr, clevel, vm):
    # Uncomment the next for disabling threading
    # Maybe due to some contention between Numexpr and Blosc?
    # bcolz.set_nthreads(bcolz.ncores//2)
    print("*** carray (using compression clevel = %d):" % clevel)
    if clevel > 0:
        x, y, z = cx, cy, cz
    t0 = time()
    cout = bcolz.eval(sexpr, vm=vm, cparams=bcolz.cparams(clevel))
    print("Time for bcolz.eval (%s) --> %.3f" % (vm, time() - t0,), end="")
    print(", cratio (out): %.1f" % (cout.nbytes / float(cout.cbytes)))
    #print "cout-->", repr(cout)


if __name__ == "__main__":

    print("Creating inputs...")

    cparams = bcolz.cparams(clevel)

    y = x.copy()
    z = x.copy()
    cx = bcolz.carray(x, cparams=cparams)
    cy = bcolz.carray(y, cparams=cparams)
    cz = bcolz.carray(z, cparams=cparams)

    for sexpr in sexprs:
        print("Evaluating '%s' with 10^%d points" % (
            sexpr, int(math.log10(N))))
        compute_ref(sexpr)
        for vm in "python", "numexpr":
            compute_carray(sexpr, clevel=0, vm=vm)
        if doprofile:
            import pstats
            import cProfile as prof
            #prof.run('compute_carray(sexpr, clevel=clevel, vm="numexpr")',
            prof.run('compute_carray(sexpr, clevel=0, vm="numexpr")',
                     #prof.run('compute_carray(sexpr, clevel=clevel,
                     # vm="python")',
                     #prof.run('compute_carray(sexpr, clevel=0, vm="python")',
                     'eval.prof')
            stats = pstats.Stats('eval.prof')
            stats.strip_dirs()
            stats.sort_stats('time', 'calls')
            stats.print_stats(20)
        else:
            for vm in "python", "numexpr":
                compute_carray(sexpr, clevel=clevel, vm=vm)



from conan.packager import ConanMultiPackager
import os

if __name__ == "__main__":
    version = os.getenv("TRAVIS_TAG") or os.getenv("APPVEYOR_REPO_TAG_NAME") or "dev"
    reference = "c-blosc/%s" % version
    upload = os.getenv("CONAN_UPLOAD") if (version != "dev") else False
    builder = ConanMultiPackager(reference=reference, upload=upload)
    builder.add_common_builds(shared_option_name="c-blosc:shared")
    builder.run()
